#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import packages
from persistent import Persistent
from zope.interface import implementer
from zope.location import Location
from zope.schema.fieldproperty import FieldProperty

# import interfaces
from pyams_content.component.association.interfaces import IAssociationParagraph, IAssociationInfo, \
    IAssociationContainer
from pyams_content.component.links.interfaces import IInternalLink
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer
from pyams_content.features.renderer.interfaces import IContentRenderer
from pyams_content.features.renderer.skin import BaseContentRenderer
from pyams_default_theme import _
from pyams_default_theme.component.association.interfaces import IAssociationParagraphRemoteContentRendererSettings
from pyams_skin.layer import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config, get_annotation_adapter


#
# Associations paragraph default renderer
#

@adapter_config(name='default', context=(IAssociationParagraph, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/association-default.pt', layer=IPyAMSLayer)
class AssociationParagraphDefaultRenderer(BaseContentRenderer):
    """Associations paragraph default renderer"""

    label = _("Default associations renderer")
    associations = ()

    i18n_context_attrs = ('title', )

    def update(self):
        super(AssociationParagraphDefaultRenderer, self).update()
        self.associations = [{'url': item.get_url(self.request),
                              'title': IAssociationInfo(item).user_title}
                             for item in IAssociationContainer(self.context).get_visible_items(self.request)]


#
# Associations paragraph remote content renderer
#

ASSOCIATION_PARAGRAPH_REMOTE_CONTENT_RENDERER_SETTINGS_KEY = 'pyams_content.association.renderer:remote-content'


@implementer(IAssociationParagraphRemoteContentRendererSettings)
class AssociationParagraphRemoteContentRendererSettings(Persistent, Location):
    """Associations paragraph remote content renderer settings"""

    display_title = FieldProperty(IAssociationParagraphRemoteContentRendererSettings['display_title'])
    display_header = FieldProperty(IAssociationParagraphRemoteContentRendererSettings['display_header'])
    factories = FieldProperty(IAssociationParagraphRemoteContentRendererSettings['factories'])
    anchors_only = FieldProperty(IAssociationParagraphRemoteContentRendererSettings['anchors_only'])


@adapter_config(context=IAssociationParagraph, provides=IAssociationParagraphRemoteContentRendererSettings)
def association_paragraph_remote_content_renderer_settings_factory(context):
    """Associations paragraph remote content renderer settings factory"""
    return get_annotation_adapter(context, ASSOCIATION_PARAGRAPH_REMOTE_CONTENT_RENDERER_SETTINGS_KEY,
                                  AssociationParagraphRemoteContentRendererSettings)


@adapter_config(name='remote-content', context=(IAssociationParagraph, IPyAMSLayer), provides=IContentRenderer)
@template_config(template='templates/association-remote-content.pt', layer=IPyAMSLayer)
class AssociationParagraphRemoteContentRenderer(BaseContentRenderer):
    """Associations container remote content renderer"""

    label = _("Include remote content")

    i18n_context_attrs = ('title', )
    links = ()

    settings_interface = IAssociationParagraphRemoteContentRendererSettings

    def update(self):
        super(AssociationParagraphRemoteContentRenderer, self).update()
        self.links = [item for item in IAssociationContainer(self.context).get_visible_items(self.request)
                      if IInternalLink.providedBy(item) and IParagraphContainerTarget.providedBy(item.target)]

    def get_renderers(self, target):
        container = IParagraphContainer(target, None)
        if container is not None:
            settings = self.settings
            for renderer in map(lambda x: x.get_renderer(self.request),
                                container.get_visible_paragraphs(None, settings.anchors_only, settings.factories)):
                if renderer is not None:
                    renderer.update()
                    yield renderer.render()
